DEFAULT_FIXTURES_DIR = "base"
TEST_FIXTURES_DIR = "test"

DEFAULT_USER_AGENT = 'Test User Agent 1.0'

CONTENT_TYPE_FIELD = "Content-Type"
CONTENT_TYPE_JSON = "application/json"
USER_AGENT_FIELD_NAME = 'HTTP_USER_AGENT'

DEFAULT_TEST_DATABASES = '__all__'
DEFAULT_TEST_SERVER_PORT = 8000

CONFIG_USER_AGENT_NAME = 'USER_AGENT_STRING'
CONFIG_AUTH_ENDPOINT = 'AUTH_ENDPOINT'

ACCESS_TOKEN_NAME = "access"
REFRESH_TOKEN_NAME = "refresh"
